@@ -36,22 +36,11 @@ import gnomemusic.utils as utils
 
 class GrlTrackerPlaylists(GObject.GObject):
 
-    METADATA_KEYS = [
-        Grl.METADATA_KEY_ALBUM,
-        Grl.METADATA_KEY_ALBUM_ARTIST,
-        Grl.METADATA_KEY_ALBUM_DISC_NUMBER,
-        Grl.METADATA_KEY_ARTIST,
+    _METADATA_PLAYLIST_KEYS = [
         Grl.METADATA_KEY_CHILDCOUNT,
         Grl.METADATA_KEY_CREATION_DATE,
-        Grl.METADATA_KEY_COMPOSER,
-        Grl.METADATA_KEY_DURATION,
-        Grl.METADATA_KEY_FAVOURITE,
         Grl.METADATA_KEY_ID,
-        Grl.METADATA_KEY_PLAY_COUNT,
-        Grl.METADATA_KEY_THUMBNAIL,
-        Grl.METADATA_KEY_TITLE,
-        Grl.METADATA_KEY_TRACK_NUMBER,
-        Grl.METADATA_KEY_URL
+        Grl.METADATA_KEY_TITLE
     ]
 
     def __init__(self, source, application, tracker_wrapper, songs_hash):
@@ -74,8 +63,9 @@ class GrlTrackerPlaylists(GObject.GObject):
         self._user_model_filter = self._coremodel.props.user_playlists_filter
         self._pls_todelete = []
         self._songs_hash = songs_hash
-        self._tracker = tracker_wrapper.props.tracker
+        self._tracker = tracker_wrapper.props.local_db
         self._tracker_wrapper = tracker_wrapper
+        self._notificationmanager = application.props.notificationmanager
         self._window = application.props.window
 
         self._user_model_filter.set_filter_func(self._user_playlists_filter)
@@ -105,41 +95,34 @@ class GrlTrackerPlaylists(GObject.GObject):
         for playlist in smart_playlists.values():
             self._model.append(playlist)
 
-        self._window.notifications_popup.push_loading()
+        self._notificationmanager.push_loading()
         query = """
         SELECT DISTINCT
-            rdf:type(?playlist)
-            tracker:id(?playlist) AS ?id
+            %(media_type)s AS ?type
+            ?playlist AS ?id
             nie:title(?playlist) AS ?title
-            tracker:added(?playlist) AS ?creation_date
-            nfo:entryCounter(?playlist) AS ?childcount
+            nrl:added(?playlist) AS ?creationDate
+            nfo:entryCounter(?playlist) AS ?childCount
         WHERE
         {
             ?playlist a nmm:Playlist .
-            OPTIONAL { ?playlist nie:url ?url;
-                       tracker:available ?available . }
-            FILTER ( !STRENDS(LCASE(?url), '.m3u')
-                     && !STRENDS(LCASE(?url), '.m3u8')
-                     && !STRENDS(LCASE(?url), '.pls')
-                     || !BOUND(nfo:belongsToContainer(?playlist)) )
-            FILTER ( !BOUND(?tag) )
-            OPTIONAL { ?playlist nao:hasTag ?tag }
         }
-        """.replace('\n', ' ').strip()
-
-        options = self._fast_options.copy()
+        """.replace("\n", " ").strip() % {
+            "media_type": int(Grl.MediaType.CONTAINER),
+        }
 
         self._source.query(
-            query, self.METADATA_KEYS, options, self._add_user_playlist)
+            query, self._METADATA_PLAYLIST_KEYS, self._fast_options,
+            self._add_user_playlist)
 
     def _add_user_playlist(
             self, source, op_id, media, remaining, data=None, error=None):
         if error:
             self._log.warning("Error: {}".format(error))
-            self._window.notifications_popup.pop_loading()
+            self._notificationmanager.pop_loading()
             return
         if not media:
-            self._window.notifications_popup.pop_loading()
+            self._notificationmanager.pop_loading()
             return
 
         playlist = Playlist(
@@ -191,9 +174,9 @@ class GrlTrackerPlaylists(GObject.GObject):
                         break
 
             self._model_filter.set_filter_func(self._playlists_filter)
-            self._window.notifications_popup.pop_loading()
+            self._notificationmanager.pop_loading()
 
-        self._window.notifications_popup.push_loading()
+        self._notificationmanager.push_loading()
         query = """
         DELETE {
             ?playlist a rdfs:Resource .
@@ -208,8 +191,7 @@ class GrlTrackerPlaylists(GObject.GObject):
         """.replace("\n", " ").strip() % {
             "playlist_id": playlist.props.pl_id
         }
-        self._tracker.update_async(
-            query, GLib.PRIORITY_LOW, None, _delete_cb, None)
+        self._tracker.update_async(query, None, _delete_cb, None)
 
     def create_playlist(self, playlist_title, callback):
         """Creates a new user playlist.
@@ -222,24 +204,25 @@ class GrlTrackerPlaylists(GObject.GObject):
             playlist_urn = result[0][0]['playlist']
             query = """
             SELECT
-                rdf:type(?playlist)
-                tracker:id(?playlist) AS ?id
+                %(media_type)s AS ?type
+                ?playlist AS ?id
                 nie:title(?playlist) AS ?title
-                tracker:added(?playlist) AS ?creation_date
-                nfo:entryCounter(?playlist) AS ?childcount
+                nrl:added(?playlist) AS ?creationDate
+                nfo:entryCounter(?playlist) AS ?childCount
                 WHERE
                 {
-                    ?playlist a nmm:Playlist .
-                    FILTER ( <%(playlist_urn)s> = ?playlist )
+                    BIND ( <%(playlist_urn)s> AS ?playlist )
                 }
-            """.replace("\n", " ").strip() % {"playlist_urn": playlist_urn}
+            """.replace("\n", " ").strip() % {
+                "media_type": int(Grl.MediaType.CONTAINER),
+                "playlist_urn": playlist_urn
+            }
 
-            options = self._fast_options.copy()
             self._source.query(
-                query, self.METADATA_KEYS, options, self._add_user_playlist,
-                callback)
+                query, self._METADATA_PLAYLIST_KEYS, self._fast_options,
+                self._add_user_playlist, callback)
 
-        self._window.notifications_popup.push_loading()
+        self._notificationmanager.push_loading()
         query = """
             INSERT {
                 _:playlist a nmm:Playlist ;
@@ -248,8 +231,7 @@ class GrlTrackerPlaylists(GObject.GObject):
                              nfo:entryCounter 0 .
             }
             """.replace("\n", " ").strip() % {"title": playlist_title}
-        self._tracker.update_blank_async(
-            query, GLib.PRIORITY_LOW, None, _create_cb, None)
+        self._tracker.update_blank_async(query, None, _create_cb, None)
 
     def check_smart_playlist_change(self):
         """Check if smart playlists need to be updated.
@@ -258,10 +240,10 @@ class GrlTrackerPlaylists(GObject.GObject):
         * it is being played (active_playlist)
         * it is visible in PlaylistsView
         """
-        active_playlist = self._coremodel.props.active_playlist
-        if (active_playlist is not None
-                and active_playlist.props.is_smart is True):
-            active_playlist.update()
+        active_core_object = self._coremodel.props.active_core_object
+        if (isinstance(active_core_object, Playlist)
+                and active_core_object.props.is_smart):
+            active_core_object.update()
         else:
             self._coremodel.emit("smart-playlist-change")
 
@@ -273,20 +255,15 @@ class Playlist(GObject.GObject):
         "playlist-loaded": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
-    METADATA_KEYS = [
+    _METADATA_PLAYLIST_KEYS = [
         Grl.METADATA_KEY_ALBUM,
-        Grl.METADATA_KEY_ALBUM_ARTIST,
-        Grl.METADATA_KEY_ALBUM_DISC_NUMBER,
         Grl.METADATA_KEY_ARTIST,
-        Grl.METADATA_KEY_CREATION_DATE,
-        Grl.METADATA_KEY_COMPOSER,
         Grl.METADATA_KEY_DURATION,
         Grl.METADATA_KEY_FAVOURITE,
         Grl.METADATA_KEY_ID,
+        Grl.METADATA_KEY_LAST_PLAYED,
         Grl.METADATA_KEY_PLAY_COUNT,
-        Grl.METADATA_KEY_THUMBNAIL,
         Grl.METADATA_KEY_TITLE,
-        Grl.METADATA_KEY_TRACK_NUMBER,
         Grl.METADATA_KEY_URL
     ]
 
@@ -329,9 +306,9 @@ class Playlist(GObject.GObject):
         self._coreselection = application.props.coreselection
         self._log = application.props.log
         self._songs_hash = songs_hash
-        self._tracker = tracker_wrapper.props.tracker
+        self._tracker = tracker_wrapper.props.local_db
         self._tracker_wrapper = tracker_wrapper
-        self._window = application.props.window
+        self._notificationmanager = application.props.notificationmanager
 
         self._fast_options = Grl.OperationOptions()
         self._fast_options.set_resolution_flags(
@@ -353,30 +330,42 @@ class Playlist(GObject.GObject):
         self._model = value
 
     def _populate_model(self):
-        self._window.notifications_popup.push_loading()
+        self._notificationmanager.push_loading()
 
         query = """
         SELECT
-            rdf:type(?song)
-            ?song AS ?tracker_urn
-            tracker:id(?song) AS ?id
-            nie:url(?song) AS ?url
-            nie:title(?song) AS ?title
-            nmm:artistName(nmm:performer(?song)) AS ?artist
-            nie:title(nmm:musicAlbum(?song)) AS ?album
-            nfo:duration(?song) AS ?duration
-            ?tag AS ?favourite
-            nie:contentAccessed(?song) AS ?last_played_time
-            nie:usageCounter(?song) AS ?play_count
+            %(media_type)s AS ?type
+            ?song AS ?id
+            ?url
+            ?title
+            ?artist
+            ?album
+            ?duration
+            ?tag AS ?favorite
+            nie:contentAccessed(?song) AS ?lastPlayed
+            nie:usageCounter(?song) AS ?playCount
         WHERE {
             ?playlist a nmm:Playlist ;
                       a nfo:MediaList ;
                         nfo:hasMediaFileListEntry ?entry .
             ?entry a nfo:MediaFileListEntry ;
                      nfo:entryUrl ?url .
-            ?song a nmm:MusicPiece ;
-                  a nfo:FileDataObject ;
-                    nie:url ?url .
+            SERVICE <dbus:%(miner_fs_busname)s> {
+                GRAPH tracker:Audio {
+                    SELECT
+                        ?song
+                        nie:title(?song) AS ?title
+                        nmm:artistName(nmm:artist(?song)) AS ?artist
+                        nie:title(nmm:musicAlbum(?song)) AS ?album
+                        nfo:duration(?song) AS ?duration
+                        ?url
+                    WHERE {
+                        ?song a nmm:MusicPiece ;
+                              nie:isStoredAs ?url .
+                        %(location_filter)s
+                    }
+                }
+            }
             OPTIONAL {
                 ?song nao:hasTag ?tag .
                 FILTER( ?tag = nao:predefined-tag-favorite )
@@ -384,16 +373,13 @@ class Playlist(GObject.GObject):
             FILTER (
                 %(filter_clause)s
             )
-            FILTER (
-                NOT EXISTS { ?song a nmm:Video }
-                && NOT EXISTS { ?song a nmm:Playlist }
-            )
-            %(location_filter)s
         }
         ORDER BY nfo:listPosition(?entry)
         """.replace('\n', ' ').strip() % {
+            "media_type": int(Grl.MediaType.AUDIO),
             "filter_clause": 'tracker:id(?playlist) = ' + self.props.pl_id,
-            "location_filter": self._tracker_wrapper.location_filter()
+            "location_filter": self._tracker_wrapper.location_filter(),
+            "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
         }
 
         def _add_to_playlist_cb(
@@ -401,7 +387,7 @@ class Playlist(GObject.GObject):
             if not media:
                 self.props.count = self._model.get_n_items()
                 self.emit("playlist-loaded")
-                self._window.notifications_popup.pop_loading()
+                self._notificationmanager.pop_loading()
                 return
 
             coresong = CoreSong(self._application, media)
@@ -414,7 +400,8 @@ class Playlist(GObject.GObject):
             Grl.ResolutionFlags.FAST_ONLY | Grl.ResolutionFlags.IDLE_RELAY)
 
         self._source.query(
-            query, self.METADATA_KEYS, options, _add_to_playlist_cb, None)
+            query, self._METADATA_PLAYLIST_KEYS, options, _add_to_playlist_cb,
+            None)
 
     def _bind_to_main_song(self, coresong):
         main_coresong = self._songs_hash[coresong.props.media.get_id()]
@@ -451,7 +438,7 @@ class Playlist(GObject.GObject):
 
         :param str new_name: new playlist name
         """
-        self._window.notifications_popup.push_loading()
+        self._notificationmanager.push_loading()
 
         def update_cb(conn, res, data):
             try:
@@ -463,7 +450,7 @@ class Playlist(GObject.GObject):
             else:
                 self._title = new_name
             finally:
-                self._window.notifications_popup.pop_loading()
+                self._notificationmanager.pop_loading()
                 self.thaw_notify()
 
         query = """
@@ -486,8 +473,7 @@ class Playlist(GObject.GObject):
         }
 
         self.freeze_notify()
-        self._tracker.update_async(
-            query, GLib.PRIORITY_LOW, None, update_cb, None)
+        self._tracker.update_async(query, None, update_cb, None)
 
     def stage_song_deletion(self, coresong, index):
         """Adds a song to the list of songs to delete
@@ -518,9 +504,9 @@ class Playlist(GObject.GObject):
         def update_cb(conn, res, data):
             # FIXME: Check for failure.
             conn.update_finish(res)
-            self._window.notifications_popup.pop_loading()
+            self._notificationmanager.pop_loading()
 
-        self._window.notifications_popup.push_loading()
+        self._notificationmanager.push_loading()
         query = """
         INSERT OR REPLACE {
             ?entry nfo:listPosition ?position .
@@ -549,7 +535,7 @@ class Playlist(GObject.GObject):
                     }
                 }
             }
-        }
+        };
         INSERT OR REPLACE {
             ?playlist nfo:entryCounter ?new_counter .
         }
@@ -564,7 +550,7 @@ class Playlist(GObject.GObject):
                     tracker:id(?playlist) = %(playlist_id)s
                 )
             }
-        }
+        };
         DELETE {
             ?playlist nfo:hasMediaFileListEntry ?entry .
             ?entry a rdfs:Resource .
@@ -583,8 +569,7 @@ class Playlist(GObject.GObject):
             "song_id": coresong.props.media.get_id()
         }
 
-        self._tracker.update_async(
-            query, GLib.PRIORITY_LOW, None, update_cb, None)
+        self._tracker.update_async(query, None, update_cb, None)
 
     def add_songs(self, coresongs):
         """Adds songs to the playlist
@@ -607,28 +592,40 @@ class Playlist(GObject.GObject):
                 return
 
             media_id = coresong.props.media.get_id()
+            miner_fs_busname = self._tracker_wrapper.props.miner_fs_busname
             query = """
             SELECT
-                rdf:type(?song)
-                ?song AS ?tracker_urn
-                tracker:id(?song) AS ?id
-                nie:url(?song) AS ?url
-                nie:title(?song) AS ?title
-                nmm:artistName(nmm:performer(?song)) AS ?artist
-                nie:title(nmm:musicAlbum(?song)) AS ?album
-                nfo:duration(?song) AS ?duration
-                ?tag AS ?favourite
-                nie:contentAccessed(?song) AS ?last_played_time
-                nie:usageCounter(?song) AS ?play_count
+                %(media_type)s AS ?type
+                ?song AS ?id
+                ?url
+                ?title
+                ?artist
+                ?album
+                ?duration
+                ?tag AS ?favorite
+                nie:contentAccessed(?song) AS ?lastPlayed
+                nie:usageCounter(?song) AS ?playCount
             WHERE {
                 ?playlist a nmm:Playlist ;
                           a nfo:MediaList ;
                             nfo:hasMediaFileListEntry ?entry .
                 ?entry a nfo:MediaFileListEntry ;
                          nfo:entryUrl ?url .
-                ?song a nmm:MusicPiece ;
-                      a nfo:FileDataObject ;
-                        nie:url ?url .
+                SERVICE <dbus:%(miner_fs_busname)s> {
+                    GRAPH tracker:Audio {
+                        SELECT
+                            ?song
+                            nie:title(?song) AS ?title
+                            nmm:artistName(nmm:artist(?song)) AS ?artist
+                            nie:title(nmm:musicAlbum(?song)) AS ?album
+                            nfo:duration(?song) AS ?duration
+                        WHERE {
+                            ?song a nmm:MusicPiece ;
+                                  nie:isStoredAs ?url .
+                            %(location_filter)s
+                        }
+                    }
+                }
                 OPTIONAL {
                     ?song nao:hasTag ?tag .
                     FILTER( ?tag = nao:predefined-tag-favorite )
@@ -636,20 +633,17 @@ class Playlist(GObject.GObject):
                 FILTER (
                    %(filter_clause)s
                 )
-                FILTER (
-                    NOT EXISTS { ?song a nmm:Video }
-                    && NOT EXISTS { ?song a nmm:Playlist }
-                )
-                %(location_filter)s
             }
             """.replace("\n", " ").strip() % {
+                "media_type": int(Grl.MediaType.AUDIO),
                 "filter_clause": "tracker:id(?song) = " + media_id,
-                "location_filter": self._tracker_wrapper.location_filter()
+                "location_filter": self._tracker_wrapper.location_filter(),
+                "miner_fs_busname": miner_fs_busname,
             }
 
-            options = self._fast_options.copy()
             self._source.query(
-                query, self.METADATA_KEYS, options, _add_to_model)
+                query, self._METADATA_PLAYLIST_KEYS, self._fast_options,
+                _add_to_model)
 
         for coresong in coresongs:
             query = """
@@ -677,7 +671,7 @@ class Playlist(GObject.GObject):
                 "song_uri": coresong.props.media.get_url()}
 
             self._tracker.update_blank_async(
-                query, GLib.PRIORITY_LOW, None, _requery_media, coresong)
+                query, None, _requery_media, coresong)
 
     def reorder(self, previous_position, new_position):
         """Changes the order of a songs in the playlist.
@@ -720,13 +714,25 @@ class Playlist(GObject.GObject):
                 "position": position
             }
             self._tracker.update_async(
-                query, GLib.PRIORITY_LOW, None, _position_changed_cb,
-                position)
+                query, None, _position_changed_cb, position)
 
 
 class SmartPlaylist(Playlist):
     """Base class for smart playlists"""
 
+    _METADATA_SMART_PLAYLIST_KEYS = [
+        Grl.METADATA_KEY_ALBUM,
+        Grl.METADATA_KEY_ALBUM_DISC_NUMBER,
+        Grl.METADATA_KEY_ARTIST,
+        Grl.METADATA_KEY_DURATION,
+        Grl.METADATA_KEY_FAVOURITE,
+        Grl.METADATA_KEY_ID,
+        Grl.METADATA_KEY_PLAY_COUNT,
+        Grl.METADATA_KEY_URL,
+        Grl.METADATA_KEY_TITLE,
+        Grl.METADATA_KEY_TRACK_NUMBER,
+    ]
+
     def __init__(self, **args):
         super().__init__(**args)
 
@@ -737,18 +743,18 @@ class SmartPlaylist(Playlist):
         if self._model is None:
             self._model = Gio.ListStore.new(CoreSong)
 
-            self._window.notifications_popup.push_loading()
+            self._notificationmanager.push_loading()
 
             def _add_to_model(source, op_id, media, remaining, error):
                 if error:
                     self._log.warning("Error: {}".format(error))
-                    self._window.notifications_popup.pop_loading()
+                    self._notificationmanager.pop_loading()
                     self.emit("playlist-loaded")
                     return
 
                 if not media:
                     self.props.count = self._model.get_n_items()
-                    self._window.notifications_popup.pop_loading()
+                    self._notificationmanager.pop_loading()
                     self.emit("playlist-loaded")
                     return
 
@@ -756,10 +762,9 @@ class SmartPlaylist(Playlist):
                 self._bind_to_main_song(coresong)
                 self._model.append(coresong)
 
-            options = self._fast_options.copy()
-
             self._source.query(
-                self.props.query, self.METADATA_KEYS, options, _add_to_model)
+                self.props.query, self._METADATA_SMART_PLAYLIST_KEYS,
+                self._fast_options, _add_to_model)
 
         return self._model
 
@@ -780,9 +785,9 @@ class SmartPlaylist(Playlist):
 
             new_model_medias.append(media)
 
-        options = self._fast_options.copy()
         self._source.query(
-            self.props.query, self.METADATA_KEYS, options, _fill_new_model)
+            self.props.query, self._METADATA_SMART_PLAYLIST_KEYS,
+            self._fast_options, _fill_new_model)
 
     def _finish_update(self, new_model_medias):
         if not new_model_medias:
@@ -822,28 +827,45 @@ class MostPlayed(SmartPlaylist):
         self._title = _("Most Played")
         self.props.query = """
         SELECT
-            rdf:type(?song)
-            tracker:id(?song) AS ?id
-            ?song AS ?tracker_urn
-            nie:title(?song) AS ?title
-            nie:url(?song) AS ?url
-            nmm:artistName(nmm:performer(?song)) AS ?artist
-            nie:title(nmm:musicAlbum(?song)) AS ?album
-            nfo:duration(?song) AS ?duration
-            nie:usageCounter(?song) AS ?play_count
-            nmm:trackNumber(?song) AS ?track_number
-            nmm:setNumber(nmm:musicAlbumDisc(?song)) AS ?album_disc_number
-            ?tag AS ?favourite
+            %(media_type)s AS ?type
+            ?song AS ?id
+            ?title
+            ?url
+            ?artist
+            ?album
+            ?duration
+            ?trackNumber
+            ?albumDiscNumber
+            ?playCount
+            ?tag AS ?favorite
         WHERE {
-            ?song a nmm:MusicPiece ;
-                    nie:usageCounter ?count .
+            SERVICE <dbus:%(miner_fs_busname)s> {
+                GRAPH tracker:Audio {
+                    SELECT
+                        ?song
+                        nie:title(?song) AS ?title
+                        nie:isStoredAs(?song) AS ?url
+                        nmm:artistName(nmm:artist(?song)) AS ?artist
+                        nie:title(nmm:musicAlbum(?song)) AS ?album
+                        nfo:duration(?song) AS ?duration
+                        nmm:trackNumber(?song) AS ?trackNumber
+                        nmm:setNumber(nmm:musicAlbumDisc(?song))
+                            AS ?albumDiscNumber
+                    WHERE {
+                        ?song a nmm:MusicPiece .
+                        %(location_filter)s
+                    }
+                }
+            }
+            ?song nie:usageCounter ?playCount
             OPTIONAL { ?song nao:hasTag ?tag .
                        FILTER (?tag = nao:predefined-tag-favorite) }
-            %(location_filter)s
         }
-        ORDER BY DESC(?count) LIMIT 50
+        ORDER BY DESC(?playCount) LIMIT 50
         """.replace('\n', ' ').strip() % {
-            "location_filter": self._tracker_wrapper.location_filter()
+            "media_type": int(Grl.MediaType.AUDIO),
+            "location_filter": self._tracker_wrapper.location_filter(),
+            "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
         }
 
 
@@ -858,27 +880,44 @@ class NeverPlayed(SmartPlaylist):
         self._title = _("Never Played")
         self.props.query = """
         SELECT
-            rdf:type(?song)
-            tracker:id(?song) AS ?id
-            ?song AS ?tracker_urn
-            nie:title(?song) AS ?title
-            nie:url(?song) AS ?url
-            nmm:artistName(nmm:performer(?song)) AS ?artist
-            nie:title(nmm:musicAlbum(?song)) AS ?album
-            nfo:duration(?song) AS ?duration
-            nie:usageCounter(?song) AS ?play_count
-            nmm:trackNumber(?song) AS ?track_number
-            nmm:setNumber(nmm:musicAlbumDisc(?song)) AS ?album_disc_number
-            ?tag AS ?favourite
+            %(media_type)s AS ?type
+            ?song AS ?id
+            ?title
+            ?url
+            ?artist
+            ?album
+            ?duration
+            ?trackNumber
+            ?albumDiscNumber
+            nie:usageCounter(?song) AS ?playCount
+            ?tag AS ?favorite
         WHERE {
-            ?song a nmm:MusicPiece ;
+            SERVICE <dbus:%(miner_fs_busname)s> {
+                GRAPH tracker:Audio {
+                    SELECT
+                        ?song
+                        nie:title(?song) AS ?title
+                        nie:isStoredAs(?song) AS ?url
+                        nmm:artistName(nmm:artist(?song)) AS ?artist
+                        nie:title(nmm:musicAlbum(?song)) AS ?album
+                        nfo:duration(?song) AS ?duration
+                        nmm:trackNumber(?song) AS ?trackNumber
+                        nmm:setNumber(nmm:musicAlbumDisc(?song))
+                            AS ?albumDiscNumber
+                    WHERE {
+                        ?song a nmm:MusicPiece .
+                        %(location_filter)s
+                    }
+                }
+            }
             FILTER ( NOT EXISTS { ?song nie:usageCounter ?count .} )
             OPTIONAL { ?song nao:hasTag ?tag .
                        FILTER (?tag = nao:predefined-tag-favorite) }
-            %(location_filter)s
         } ORDER BY nfo:fileLastAccessed(?song) LIMIT 50
         """.replace('\n', ' ').strip() % {
-            "location_filter": self._tracker_wrapper.location_filter()
+            "media_type": int(Grl.MediaType.AUDIO),
+            "location_filter": self._tracker_wrapper.location_filter(),
+            "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
         }
 
 
@@ -900,30 +939,47 @@ class RecentlyPlayed(SmartPlaylist):
             time.gmtime(time.time() - seconds_difference))
         self.props.query = """
         SELECT
-            rdf:type(?song)
-            tracker:id(?song) AS ?id
-            ?song AS ?tracker_urn
-            nie:title(?song) AS ?title
-            nie:url(?song) AS ?url
-            nmm:artistName(nmm:performer(?song)) AS ?artist
-            nie:title(nmm:musicAlbum(?song)) AS ?album
-            nfo:duration(?song) AS ?duration
-            nie:usageCounter(?song) AS ?play_count
-            nmm:trackNumber(?song) AS ?track_number
-            nmm:setNumber(nmm:musicAlbumDisc(?song)) AS ?album_disc_number
-            ?tag AS ?favourite
+            %(media_type)s AS ?type
+            ?song AS ?id
+            ?title
+            ?url
+            ?artist
+            ?album
+            ?duration
+            ?trackNumber
+            ?albumDiscNumber
+            nie:usageCounter(?song) AS ?playCount
+            ?tag AS ?favorite
         WHERE {
-            ?song a nmm:MusicPiece ;
-                    nie:contentAccessed ?last_played .
-            FILTER ( ?last_played > '%(compare_date)s'^^xsd:dateTime
+            SERVICE <dbus:%(miner_fs_busname)s> {
+                GRAPH tracker:Audio {
+                    SELECT
+                        ?song
+                        nie:title(?song) AS ?title
+                        nie:isStoredAs(?song) AS ?url
+                        nmm:artistName(nmm:artist(?song)) AS ?artist
+                        nie:title(nmm:musicAlbum(?song)) AS ?album
+                        nfo:duration(?song) AS ?duration
+                        nmm:trackNumber(?song) AS ?trackNumber
+                        nmm:setNumber(nmm:musicAlbumDisc(?song))
+                            AS ?albumDiscNumber
+                    WHERE {
+                        ?song a nmm:MusicPiece .
+                        %(location_filter)s
+                    }
+                }
+            }
+            ?song nie:contentAccessed ?lastPlayed .
+            FILTER ( ?lastPlayed > '%(compare_date)s'^^xsd:dateTime
                      && EXISTS { ?song nie:usageCounter ?count .} )
             OPTIONAL { ?song nao:hasTag ?tag .
                        FILTER (?tag = nao:predefined-tag-favorite) }
-            %(location_filter)s
-        } ORDER BY DESC(?last_played) LIMIT 50
+        } ORDER BY DESC(?lastPlayed) LIMIT 50
         """.replace('\n', ' ').strip() % {
+            "media_type": int(Grl.MediaType.AUDIO),
             'compare_date': compare_date,
-            "location_filter": self._tracker_wrapper.location_filter()
+            "location_filter": self._tracker_wrapper.location_filter(),
+            "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
         }
 
 
@@ -945,29 +1001,47 @@ class RecentlyAdded(SmartPlaylist):
             time.gmtime(time.time() - seconds_difference))
         self.props.query = """
         SELECT
-            rdf:type(?song)
-            tracker:id(?song) AS ?id
-            ?song AS ?tracker_urn
-            nie:title(?song) AS ?title
-            nie:url(?song) AS ?url
-            nmm:artistName(nmm:performer(?song)) AS ?artist
-            nie:title(nmm:musicAlbum(?song)) AS ?album
-            nfo:duration(?song) AS ?duration
-            nie:usageCounter(?song) AS ?play_count
-            nmm:trackNumber(?song) AS ?track_number
-            nmm:setNumber(nmm:musicAlbumDisc(?song)) AS ?album_disc_number
-            ?tag AS ?favourite
+            %(media_type)s AS ?type
+            ?song AS ?id
+            ?title
+            ?url
+            ?artist
+            ?album
+            ?duration
+            ?trackNumber
+            ?albumDiscNumber
+            nie:usageCounter(?song) AS ?playCount
+            ?tag AS ?favorite
         WHERE {
-            ?song a nmm:MusicPiece ;
-                    tracker:added ?added .
-            FILTER ( tracker:added(?song) > '%(compare_date)s'^^xsd:dateTime )
+            SERVICE <dbus:%(miner_fs_busname)s> {
+                GRAPH tracker:Audio {
+                    SELECT
+                        ?song
+                        nie:title(?song) AS ?title
+                        nie:isStoredAs(?song) AS ?url
+                        nmm:artistName(nmm:artist(?song)) AS ?artist
+                        nie:title(nmm:musicAlbum(?song)) AS ?album
+                        nfo:duration(?song) AS ?duration
+                        nmm:trackNumber(?song) AS ?trackNumber
+                        nmm:setNumber(nmm:musicAlbumDisc(?song))
+                            AS ?albumDiscNumber
+                        ?added
+                    WHERE {
+                        ?song a nmm:MusicPiece ;
+                              nrl:added ?added .
+                        %(location_filter)s
+                        FILTER ( ?added > '%(compare_date)s'^^xsd:dateTime )
+                    }
+                }
+            }
             OPTIONAL { ?song nao:hasTag ?tag .
                        FILTER (?tag = nao:predefined-tag-favorite) }
-            %(location_filter)s
-        } ORDER BY DESC(tracker:added(?song)) LIMIT 50
+        } ORDER BY DESC(nrl:added(?song)) LIMIT 50
         """.replace('\n', ' ').strip() % {
+            "media_type": int(Grl.MediaType.AUDIO),
             'compare_date': compare_date,
-            "location_filter": self._tracker_wrapper.location_filter()
+            "location_filter": self._tracker_wrapper.location_filter(),
+            "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
         }
 
 
@@ -982,27 +1056,41 @@ class Favorites(SmartPlaylist):
         self._title = _("Favorite Songs")
         self.props.query = """
             SELECT
-                rdf:type(?song)
-                tracker:id(?song) AS ?id
-                ?song AS ?tracker_urn
-                nie:title(?song) AS ?title
-                nie:url(?song) AS ?url
-                nmm:artistName(nmm:performer(?song)) AS ?artist
-                nie:title(nmm:musicAlbum(?song)) AS ?album
-                nfo:duration(?song) AS ?duration
-                nie:usageCounter(?song) AS ?play_count
-                nmm:trackNumber(?song) AS ?track_number
-                nmm:setNumber(nmm:musicAlbumDisc(?song)) AS ?album_disc_number
-                nao:predefined-tag-favorite AS ?favourite
+                %(media_type)s AS ?type
+                ?song AS ?id
+                ?title
+                ?url
+                ?artist
+                ?album
+                ?duration
+                ?trackNumber
+                ?albumDiscNumber
+                nie:usageCounter(?song) AS ?playCount
+                nao:predefined-tag-favorite AS ?favorite
             WHERE {
-                ?song a nmm:MusicPiece ;
-                        nie:isStoredAs ?as ;
-                        nao:hasTag nao:predefined-tag-favorite .
-                ?as nie:url ?url .
-                OPTIONAL { ?song nao:hasTag ?tag .
-                           FILTER (?tag = nao:predefined-tag-favorite) }
-                %(location_filter)s
-            } ORDER BY DESC(tracker:added(?song))
+                SERVICE <dbus:%(miner_fs_busname)s> {
+                    GRAPH tracker:Audio {
+                        SELECT
+                            ?song
+                            nie:title(?song) AS ?title
+                            nie:isStoredAs(?song) AS ?url
+                            nmm:artistName(nmm:artist(?song)) AS ?artist
+                            nie:title(nmm:musicAlbum(?song)) AS ?album
+                            nfo:duration(?song) AS ?duration
+                            nmm:trackNumber(?song) AS ?trackNumber
+                            nmm:setNumber(nmm:musicAlbumDisc(?song))
+                                AS ?albumDiscNumber
+                            nrl:added(?song) AS ?added
+                        WHERE {
+                            ?song a nmm:MusicPiece .
+                            %(location_filter)s
+                        }
+                    }
+                }
+                ?song nao:hasTag nao:predefined-tag-favorite .
+            } ORDER BY DESC(?added)
         """.replace('\n', ' ').strip() % {
-            "location_filter": self._tracker_wrapper.location_filter()
+            "media_type": int(Grl.MediaType.AUDIO),
+            "location_filter": self._tracker_wrapper.location_filter(),
+            "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
         }
